# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - AddLinkToPage

    Add a link using a form in the page, getting details of the linked document
    and inserting them with the link itself.

    @copyright: 2010 Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL, see COPYING for details.
"""

Dependencies = ['pages']

from MoinMoin.action import ActionBase
from MoinMoin.PageEditor import PageEditor
from MoinContentSupport import ActionSupport
import re

macro_pattern = re.compile(ur'^(?P<leading>.*?)<<AddLinkToPage\((?P<identifier>[^\s,)]+).*?\)>>(?P<trailing>.*)$',
    re.MULTILINE | re.UNICODE)

# Action class and supporting functions.

class AddLinkToPage(ActionBase, ActionSupport):

    "An action adding links to pages."

    def do_action(self):

        "Create the new event."

        _ = self._
        form = self.get_form()

        # If no title exists in the request, an error message is returned.

        identifier = form.get("identifier", [None])[0]

        if not identifier:
            return 0, _("No identifier specified.")

        link = form.get("link", [None])[0]

        if not link:
            return 0, _("No link specified.")

        return self.add_link_to_page(identifier, link)

    def render_success(self, msg, msgtype=None):

        """
        Render neither 'msg' nor 'msgtype' since redirection should occur
        instead.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

    def add_link_to_page(self, identifier, link):

        """
        For the macro with the given 'identifier', add 'link' to the current
        page.
        """

        _ = self._
        request = self.request
        page = self.page
        formatter = request.formatter
        form = self.get_form()

        # Get the link details.

        title = form.get('title', [link])[0]
        introduction = form.get('introduction', [""])[0]
        description = form.get('description', [""])[0]
        insert_before = form.get('insert_before', [""])[0]

        # Encode the link details.
        # NOTE: Should support different formatting options.

        link_details = "%s[[%s%s]]%s" % (
            introduction and ('"%s" ' % formatter.escapedText(introduction)) or "",
            link,
            title and ('|%s' % title) or "",
            description and (" - ''%s''" % description) or ""
            )

        # Open the page for editing.

        new_page = PageEditor(request, page.page_name)

        # Parse the page.

        page_body = page.get_raw_body()

        for match in macro_pattern.finditer(page_body):
            macro_identifier = match.group("identifier")
            leading_text = match.group("leading")
            trailing_text = match.group("trailing")
            start, end = match.span()

            # Where this identifier matches this macro's identifier, insert the
            # link details.

            if macro_identifier == identifier:
                if insert_before:
                    page_body = page_body[:start] + leading_text + link_details + trailing_text + "\n" + page_body[start:]
                else:
                    page_body = page_body[:end] + "\n" + leading_text + link_details + trailing_text + page_body[end:]

                # Save the new version of the page.

                new_page.saveText(page_body, 0)
                break

        # NOTE: Perhaps show a message upon failure.

        request.http_redirect(page.url(request))
        return 1, None

# Action function.

def execute(pagename, request):
    AddLinkToPage(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
