# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - AddLinkToPage

    Add a link using a form in the page, getting details of the linked document
    and inserting them with the link itself.

    @copyright: 2010 Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL, see COPYING for details.
"""

Dependencies = ["pages"]

from MoinMoin import wikiutil
from MoinMoin.PageEditor import PageEditor
from MoinContentSupport import get_form
import urllib
import re

title_pattern = re.compile(ur'<(?P<tag>title|h\d)(\s.*?)?>(?P<title>.*?)</(?P=tag)>', re.MULTILINE | re.UNICODE | re.DOTALL)
paragraph_pattern = re.compile(ur'<p(\s.*?)?>(?P<text>.*?)(?=<p(\s.*?)?>|</p>)', re.MULTILINE | re.UNICODE | re.DOTALL)
tag_pattern = re.compile(ur'<.*?>', re.MULTILINE | re.UNICODE | re.DOTALL)

def get_link_info(link):

    "Get information from the given 'link'."

    # NOTE: Insist on remote URLs!

    try:
        f = urllib.urlopen(link)
    except IOError:
        return None

    try:
        s = f.read()
        first_title = ""

        for title_match in title_pattern.finditer(s):
            title = title_match.group("title").strip()
            start, end = title_match.span()

            if not first_title:
                first_title = title

            for intro_match in paragraph_pattern.finditer(s[end:]):
                intro = get_flattened_content(intro_match.group("text")).strip()
                if intro:
                    return title, intro
    finally:
        f.close()

    return first_title, ""

def get_flattened_content(s):

    "Get HTML or XHTML without the tags."

    l = []
    last = 0
    for match in tag_pattern.finditer(s):
        start, end = match.span()
        l.append(s[last:start])
        last = end
    l.append(s[last:])
    return "".join(l).replace("\n", " ")

# Macro functions.

def execute(macro, args):

    """
    Execute the 'macro' with the given 'args': 
    """

    request = macro.request
    formatter = macro.formatter
    page = formatter.page
    form = get_form(request)
    _ = macro._

    output = []

    # Interpret the arguments.

    try:
        parsed_args = args and wikiutil.parse_quoted_separated(args, name_value=False) or []
    except AttributeError:
        parsed_args = args.split(",")

    parsed_args = [arg for arg in parsed_args if arg]

    # The macro's identifier should always appear first.

    identifier = parsed_args[0]

    # Look for keywords determining the action of the macro.

    insert_before = "before" in parsed_args[1:] or not ("after" in parsed_args[1:])

    # If the request refers to this macro, another kind of form will be shown.

    active_identifier = form.get('add_link_to_page', [None])[0]
    link = form.get('add_link_to_page_link', [None])[0]

    # Test for usage of this macro.
    # Where this macro is not active, don't do anything.

    if identifier != active_identifier:
        show_macro = 1

    # Otherwise, show a form containing link details.

    else:

        # Acquire information from the link.

        link_info = get_link_info(link)

        # NOTE: Perhaps show a message upon success/failure.

        if link_info is None:
            show_macro = 1

        # Information was retrieved and is now shown.

        else:
            title, introduction = link_info

            # Show a form containing the retrieved information suitable for the
            # corresponding action.

            output.append(u'<form class="macro" method="POST" action="%s/%s">' % (
                request.getScriptname(), wikiutil.quoteWikinameURL(page.page_name)))

            output.append(u'''
    <input type="hidden" name="identifier" value="%(identifier)s" />
    <input type="hidden" name="doit" value="1" />
    <input type="hidden" name="insert_before" value="%(insert_before)s" />
    <input type="hidden" name="action" value="AddLinkToPage" />''' % {
                "identifier" : wikiutil.escape(identifier, 1),
                "insert_before" : insert_before and "true" or ""
                })

            output.append(u'''
    <table>
      <tr>
        <th>%(url_label)s</th>
        <td><input type="text" name="link" value="%(link)s" /></td>
      </tr>
      <tr>
        <th>%(title_label)s</th>
        <td><input type="text" name="title" value="%(title)s" /></td>
      </tr>
      <tr>
        <th>%(intro_label)s</th>
        <td><input type="text" name="introduction" value="%(intro)s" /></td>
      </tr>
      <tr>
        <th>%(description_label)s</th>
        <td><input type="text" name="description" /></td>
      </tr>
      <tr>
        <td colspan="2"><input type="submit" value="%(submit_label)s" /></td>
      </tr>
    </table>''' % {
                "link" : wikiutil.escape(link, 1),
                "title" : wikiutil.escape(title, 1),
                "intro" : wikiutil.escape(introduction, 1),
                "url_label" : wikiutil.escape(_("URL")),
                "title_label" : wikiutil.escape(_("Title")),
                "intro_label" : wikiutil.escape(_("Introduction")),
                "description_label" : wikiutil.escape(_("Description")),
                "submit_label" : wikiutil.escape(_("Submit link"))
                })

            output.append(u'</form>')

            # Don't show the macro.

            show_macro = 0

    # If the macro is to be shown, emit the usual fields.

    if show_macro:
        output.append(u'<form class="macro" method="POST" action="%s/%s">' % (
            request.getScriptname(), wikiutil.quoteWikinameURL(page.page_name)))
        output.append(u'<input type="hidden" name="add_link_to_page" value="%s" />' %
            wikiutil.escape(identifier, 1))
        output.append(u'<div>')
        output.append(u'<input type="text" name="add_link_to_page_link" />')
        output.append(u'<input type="submit" value="%s" />' %
            wikiutil.escape(_("Add link")))
        output.append(u'</div>')
        output.append(u'</form>')

    return formatter.rawHTML('\n'.join(output))

# vim: tabstop=4 expandtab shiftwidth=4
